import json


class Response:
    def __init__(self, status, data):
        self.status = str(status)
        self.raw_data = data

    @property
    def data(self):
        return str.encode(json.dumps(self.raw_data))



class Request:
    def __init__(self, environ):
        self.method = environ['REQUEST_METHOD']
        self.path = environ['PATH_INFO']
        self.data = environ['wsgi.input']
        self.environ = environ
        self.content_type = environ.get('CONTENT_TYPE')
        self.auth_header = environ.get('HTTP_AUTHORIZATION')



import json
import cgi
from .fields import FileField


def parse_json(data):
    return json.loads(str(data.read(), 'utf-8'))


def parse_form(data, request):
    content_type, pdict = cgi.parse_header(request.content_type)
    pdict['boundary'] = bytes(pdict['boundary'], 'utf-8')
    form = cgi.parse_multipart(data, pdict)
    for key, value in form.items():
        for i in range(len(value)):
            if type(value[i]) == bytes:
                value[i] = FileField(value[i])
        if len(value) == 1:
            form[key] = value[0]
    return form



from setuptools import setup
from os.path import join, dirname


setup(name='myapiframework',
      version='1.0',
      description='eazy python framework',
      long_description=open(join(dirname(__file__), 'README.md')).read(),
      long_description_content_type = "text/markdown",
      install_requires = ['parse', 'gunicorn'],
      packages=['myapiframework'],
      author='Barashov',
      author_email='barashovmisha@gmail.com',
      url='https://github.com/Barashov/Myapi'
)




from .request import Request
from parse import parse
from .exceptions import HandlerWasNotFoundException
from .backends import BaseBackend


class RoutersConstructor:
    def __init__(self) -> None:
        self._routers = {'GET': {},
                         'POST': {},
                         'PUT': {},
                         'PATCH': {}}
        
    def get(self, path):
        def wrapper(handler):
            self._routers['GET'][path] = handler
        return wrapper

    def post(self, path):
        def wrapper(handler):
            self._routers['POST'][path] = handler
        return wrapper

    def put(self, path):
        def wrapper(handler):
            self._routers['PUT'][path] = handler
        return wrapper
    
    def patch(self, path):
        def wrapper(handler):
            self._routers['PATCH'][path] = handler
        return wrapper


class MyAPI(RoutersConstructor):
    authentication_class = BaseBackend

    def __init__(self):
        super(MyAPI, self).__init__()

    def __call__(self, environ: dict, start_response):
        request = Request(environ)
        request.user = self.authentication_class.authenticate(request)
        data = self.request_handle(request, start_response)
        return iter([data])

    def get_handler(self, request_path, method):
        for path, handler in self._routers[method].items():
            parse_result = parse(path, request_path)
            if parse_result is not None:
                return parse_result.named, handler
        raise HandlerWasNotFoundException

    def request_handle(self, request, start_response):
        try:
            request_data, handler = self.get_handler(request.path, request.method)
        except HandlerWasNotFoundException:
            start_response('404', [])
            return b'page not found'

        result = handler(request, **request_data)
        start_response(result.status, [])
        return result.data

